unit WordReport;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DsgnIntf;

type
  TWordReport = class;

  TReportEvent = procedure(Sender: TWordReport; var Word: OleVariant) of object;

  TAboutWordReport = class
  end;

  TWordReport = class(TComponent)
  private
    { Private declarations }
    word, doc:OleVariant;
    FTemplateName:string;
    FMaster, FDetail:TDataSet;
    FReportHeaderPrint:TReportEvent;
    FReportFooterPrint:TReportEvent;
    FPageHeaderPrint:TReportEvent;
    FDetailPrint:TReportEvent;
    FPageFooterPrint:TReportEvent;
    FPreview:boolean;
    FAboutWordReport:TAboutWordReport;
    procedure DoDetail;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    procedure Execute;
  published
    { Published declarations }
    property TemplateName:string read FTemplateName write FTemplateName;
    property MasterDataset:TDataSet read FMaster write FMaster;
    property DetailDataset:TDataSet read FDetail write FDetail;
    property ReportHeaderPrint:TReportEvent read FReportHeaderPrint write FReportHeaderPrint;
    property ReportFooterPrint:TReportEvent read FReportFooterPrint write FReportFooterPrint;
    property PageHeaderPrint:TReportEvent read FPageHeaderPrint write FPageHeaderPrint;
    property DetailPrint:TReportEvent read FDetailPrint write FDetailPrint;
    property PageFooterPrint:TReportEvent read FPageFooterPrint write FPageFooterPrint;
    property Preview:boolean read FPreview write FPreview;
    property About:TAboutWordReport read FAboutWordReport write FAboutWordReport;
  end;

  TAboutWordReportProp=class (TPropertyEditor)
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

procedure Register;

implementation

uses ComObj, uAbout, uWait;

procedure Register;
begin
  RegisterComponents('Mohan', [TWordReport]);
  RegisterPropertyEditor(TypeInfo(TAboutWordReport), nil,
    '', TAboutWordReportProp)
end;

function TAboutWordReportProp.GetAttributes: TPropertyAttributes;
begin
  Result:=[paDialog, paReadOnly];
end;

procedure TAboutWordReportProp.Edit;
begin
  with TAboutBox.Create(Application) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

constructor TWordReport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then
  begin
    word:=CreateOleObject('word.application');
  end;
end;

destructor TWordReport.Destroy;
begin
  if not (csDesigning in ComponentState) then
    word.Quit(false);
  inherited Destroy;
end;

procedure TWordReport.DoDetail;
var
  i, vcount:integer;
  table:OleVariant;
  field:TField;
begin
  table:=doc.Tables.Item(1);
  with FDetail do
  begin
    First;
    while not eof do
    begin
      vcount:=0;
      for i := 0 to FieldCount-1 do
      begin
        field:=Fields[i];
        if field.Visible then
        begin
          Inc(vcount);
          table.Cell(RecNo, vcount).Range.Text:=field.DisplayText;
        end;
      end;

      if Assigned(FDetailPrint) then
        FDetailPrint(self, word);
      Next;
      table.Rows.Add;
    end;
  end;
end;

procedure TWordReport.Execute;
var
  v, ff:OleVariant;
  field:TField;
  i:integer;
begin
  fWait:=TfWait.Create(self);
  if FPreview then
    Word.Visible:=true
  else
    fWait.Visible:=True;
  if Assigned(FReportHeaderPrint) then
    FReportHeaderPrint(self, word);
  with FMaster do
  begin
    First;
    while not eof do
    begin
      Application.ProcessMessages;
      if fWait.Stop then
        break;
      v:=FTemplateName;
      Word.Documents.Add(v);
      doc:=Word.ActiveDocument;

      ff:=doc.FormFields;
      for i := 0 to FieldCount-1 do
      begin
        field:=Fields[i];
        if field.Visible then
        begin
          v:=field.FieldName;
          ff.Item(v).Result:=field.DisplayText;
        end;
      end;

      if Assigned(FPageHeaderPrint) then
        FPageHeaderPrint(self, word);
      if Assigned(FDetail) then
        DoDetail;
      if Assigned(FPageFooterPrint) then
        FPageFooterPrint(self, word);
      if FPreview then
      begin
        doc.PrintPreview;     //preview=on
        if MessageDlg('Continue?', mtConfirmation, [mbYes, mbNo], 0)=mrNo then
        begin
          doc.ClosePrintPreview;
          doc.Close(false);
          break;
        end;
        doc.ClosePrintPreview;
      end
      else
      begin
        doc.PrintOut(false);
      end;
      doc.Close(false);
      Next;
    end;
  end;
  if Assigned(FReportFooterPrint) then
    FReportFooterPrint(self, word);
  fWait.Free;
end;

end.
